<?php
/**
 * Live CSS application component.
 */
class LiveCss extends CApplicationComponent
{
	
	private $_assetsUrl;
	
	/**
	 * Base path of site and framework
	 * @var unknown
	 */
	public $basePath;
	
	public $theme;
	public $hideModules;
	public $useLess;
	public $paths;
	
	/**
	 * Initializes the component.
	 */
	public function init()
	{
		if (Yii::getPathOfAlias('livecss') === false)
			Yii::setPathOfAlias('livecss', dirname(__FILE__));
		
		Yii::app()->clientScript->registerScriptFile($this->getAssetsUrl().'/js/livecss_plugin.js');
		
		if (YII_DEBUG) Yii::app()->clientScript->registerScriptFile($this->getAssetsUrl().'/js/ace/src/ace-uncompressed.js');
		else Yii::app()->clientScript->registerScriptFile($this->getAssetsUrl().'/js/ace/src/ace.js');
		
		Yii::app()->clientScript->registerScriptFile($this->getAssetsUrl().'/js/ace/src/mode-css.js');
		Yii::app()->clientScript->registerScriptFile($this->getAssetsUrl().'/js/ace/src/theme-twilight.js');
		
		Yii::app()->clientScript->registerCssFile($this->getAssetsUrl().'/css/livecss.css');
		
		if ($this->useLess) {
			Yii::app()->clientScript->registerScriptFile($this->getAssetsUrl().'/js/ace/src/mode-less.js');
			Yii::app()->clientScript->registerScriptFile($this->getAssetsUrl().'/js/ace/src/mode-less_highlight_rules.js');
			Yii::app()->clientScript->registerScriptFile($this->getAssetsUrl().'/js/less-1.3.0.min.js');
		}
		
		parent::init();
	}
	
	
	public function saveSettingsToJavascript() {
		// create settings
		$assets = array_flip(Yii::app()->getAssetManager()->getAssets());
		
		// stylesheets
		$stylesheets = $this->searchStylesheets('css');
		$stylesheets = array_merge($stylesheets, $this->searchStylesheets('less'));
		$stylesheets = array_merge($stylesheets, $this->paths);
		
		$js = CJavaScript::encode(array(
				'theme' => $this->theme,
				'baseUrl' => Yii::app()->baseUrl.'/',
				'useLess' => $this->useLess,
				'assets' => $assets,
				'stylesheets' => $stylesheets,
		));
		
		Yii::app()->clientScript->registerScript('InitLiveCss', 'LiveCss = {"settings" : null}; LiveCss.settings = ' . $js, CClientScript::POS_LOAD);
	}
	
	protected function searchStylesheets($type) {
		$directory = Yii::getPathOfAlias('webroot').DIRECTORY_SEPARATOR.$type;
		$files = scandir($directory);
		
		$filtered = array();
		foreach ($files as $file) {
			if (LiveCss::endsWith($file, $type)) $filtered[$file] = $directory.DIRECTORY_SEPARATOR.$file;
		}
		
		return $filtered;
	}
	
	/**
	 * Returns the URL to the published assets folder.
	 * @return string the URL
	 */
	protected function getAssetsUrl()
	{
		if (isset($this->_assetsUrl))
			return $this->_assetsUrl;
		else
		{
			$assetsPath = Yii::getPathOfAlias('livecss.assets');
			$assetsUrl = Yii::app()->assetManager->publish($assetsPath, false, -1, YII_DEBUG);
			return $this->_assetsUrl = $assetsUrl;
		}
	}
	
	private static function endsWith($haystack, $needle)
	{
	    $length = strlen($needle);
	    if ($length == 0) {
	        return true;
	    }
	
	    return (substr($haystack, -$length) === $needle);
	}
}